<?php
/********************************************************
* The source code is the source code of a free software *
* and may be used under the term of the general public	*
* license (GPL) of the free software fundation. 	*
*							*
* Author: Pablo Yanez Trujillo  			*
* Filename: scripts/class.Template.php			*
* Date:	28.05.04					*
*********************************************************/

/* 
    File description:
    Creates a class that parse templates
*/

	

define("FILE_NOT_FOUND", 1);
define("FILE_NO_ACCESS", 2);
define("WRONG_TEMPLATE", 3);
define("PARSE_ERROR", 4);
define("WRONG_ASSIGN", 5);
define("WRONG_ORDER", 6);


$err_db = array(
	FILE_NOT_FOUND 	=> "file not found",
	FILE_NO_ACCESS	=> "access denied",
	WRONG_TEMPLATE	=> "your templates array is wrong or you didn't pass an array",
	PARSE_ERROR	=> "errors while parsing",
	WRONG_ASSIGN	=> "you have used the assign method bad",
	WRONG_ORDER	=> "you must use set_templates before using change_templates"
      );

/******************************************************
 **
 * Class EasyTemplate
 *
 * Synopsis: This class reads template file (.tpl) and 
 * parse them with the text you set on it. This is very
 * useful, when you want to separate your php code from
 * the layout of the website.
 *
 * You are able to create indepent scripts (login procedure,
 * and so on) without thinking about the layout. 
 *
 * @author              Pablo Yanez Trujillo
 * @copyright   	(C) Pablo Yanez Trujillo, 2004
 * @version             0.0.2
********************************************************/

class EasyTemplate 
{
	/**
	 * Contains the whole body of the template
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_body = "";
	/**
	 * Contains the directory where the templates
	 * are saved
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_location = "";
	/**
 	 * Array with filelist
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_filelist = array();
	/**
	 * Array with the variable to be parsed
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_parse_vars = array();
	/**
	 * last error id. if 0 no error
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_err_number = 0;
	/**
	 * last error message, if $this->tpl_parse_vars=0
	 * then the string is empty
	 * @var		mixed
	 * @access	private
	 */
	var $tpl_err_msg="";

	/**
	*
	*
	* @access	private
	*/
	function check_for_errors()
	{
		global $err_db;
		if ($this->tpl_err_number) {
			$msg = "Error using QuickTemplate class<p>\n";
			$msg = "\"" . $this->tpl_err_msg . "\", ";
			$msg .= "exiting with error <b>" . $this->tpl_err_number;
			$msg .= "</b>: <i>" . $err_db[$this->tpl_err_number] ."</i>\n";
			die($msg);
		}
	}

	/**
	* @access	private
	*/
	function clear_err_flags()
	{
		$this->tpl_err_number=0;
		$this->tpl_err_msg="";
	}

	/**
	*
	* Creates a new template. If  path is not passed
	* QuickTemplate uses the current directory.
	*
	* @return	mixed	 Description
	* @access	public
	*/
	function EasyTemplate($path=".") {
		if ($path[strlen($path)-1]!="/") $path .= "/";
		$this->tpl_location=$path;
	}

	/**
	*
	* Pass an array with the alias and only the file name
	* of the template. Example
	* $tpl->set_templates(array('main' => 'main.tpl'));
	*
	* @access	public
	*/
	function set_templates($templates_array) {
		$this->clear_err_flags();
		if ($templates_array==array()) {
			$this->tpl_err_number=WRONG_TEMPLATE;
			$this->tpl_err_msg="You passed an empty array";
		} elseif (!is_array($templates_array)) {
			$this->tpl_err_number=WRONG_TEMPLATE;
			$this->tpl_err_msg="You have to use an array";
		} else {
			$this->tpl_filelist=$templates_array;
		}
		$this->check_for_errors();
	}

	/**
	*
	* The array you may pass have to have the same format as the one of the
	* set_templates function. You can change with this function the files of 
	* your templates and even add new templates. Take care of the key names
	* because if you use key, that has been already used, the key will be 
	* overwritten by the new value. You can use this function only when you used
	* set_templates at least one time.
	* 
	* @return	nothing
	* @access	public
	* @see		set_templates
	*/
	function change_templates($templates_array)  
	{
		$this->clear_err_flags();
		if ($templates_array==array()) {
                        $this->tpl_err_number=WRONG_TEMPLATE;
                        $this->tpl_err_msg="You passed an empty array";
		} elseif (!is_array($templates_array)) {
                        $this->tpl_err_number=WRONG_TEMPLATE;
                        $this->tpl_err_msg="You have to use an array";
		} elseif ($this->tpl_filelist==array())
		{
			$this->tpl_err_number=WRONG_ORDER;
                        $this->tpl_err_msg="Wrong order by calling the functions";
		} else {
			$this->tpl_filelist = array_merge($this->tpl_filelist, $templates_array);
		}
		
		$this->check_for_errors();
	}
	

	/**
	*
	* Set the $content of $varname of $alias template.
	* If $add=1, then you will add the new content to the old content.
	* When you parse the contents will be deleted
	*
	* @return	array	 Description
	* @access	public
	* @see		set_templates
	*/

	function assign($alias, $varname="", $content="", $add=1)
	{
	    /* This function checks first whether $alias is an array.
	       If so, then the new assign version is used, varname, content and add must
	       be blank. */
	       $this->clear_err_flags();

		if (is_array($alias)) {
			if($varname!="" or $content!="" or $add != 1) {
				$this->tpl_err_number=WRONG_ASSIGN;
	                        $this->tpl_err_msg="All other arguments must be blank";
			} else {
				$this->__assign_a($alias);
			}
		} else /* old function */
		$this->__assign_t($alias, $varname, $content, $add);
		$this->check_for_errors();	    
	}

	/**
	* @access	private
	*/
	function __assign_a($arr)
	{	
		if ($arr==array()) { 
			$this->tpl_err_number=WRONG_ASSIGN;
                        $this->tpl_err_msg="This is blank array";
			return;
		}

		foreach($arr as $array => $value)
		{
			$alias = $value['alias'];
			unset($value['alias']);
			$add = isset($value['add']) ? $value['add'] : 0;
			unset($value['add']);
			foreach($value as $key => $val)
			{
				$this->__assign_t($alias, $key, $val, $add);
			}
		}
		
	}

	/**
        * @access       private
        */
	function __assign_t($alias, $varname, $content, $add) {
		/*echo "<pre>\n";
		echo "\$this->tpl_parse_vars\n";
		print_r($this->tpl_parse_vars);
		echo "</pre>\n"; debug information only! */
		$this->clear_err_flags();
		if (!isset($this->tpl_parse_vars[$alias][$varname]))
			$this->tpl_parse_vars[$alias][$varname]="";
		
		if ($add) {
			$this->tpl_parse_vars[$alias][$varname] .= $content;
		} else {
			$this->tpl_parse_vars[$alias][$varname] = $content;
		}
	}
	
	/**
	*
	* returns the body that was parsed
	*
	* @return	mixed	 Description
	* @access	public
	* @see		parse
	*/
	function get_body() {
		$this->clear_err_flags();
		return $this->tpl_body;
	}

	/**
	*
	* If you want to get the content of a template file
	* you have to parse the template. $alias is the alias
	* name of a template. You have to call set_template and
	* assign before parse. Parse open the template file and then
	* search for variables and change their values. If the alias
	* wasn't assigned before, then parse won't be able to work.
	* You can get the parse text with the function get_body().
	* When the parse process is done, the temporal variable with
	* the parse temprals will be deleted. When you parse again,
	* you will loose the old content.
	*
	* @return	mixed	 Description
	* @access	public
	* @see		assign,get_body
	*/
	function parse($alias, $clear=1) {
		$this->clear_err_flags();
		/* check whether assign was made */
		if (!isset($this->tpl_parse_vars[$alias])) {
			/* template contains only text */
			$this->tpl_parse_vars=array_merge($this->tpl_parse_vars,array($alias => array()));
			/*$this->tpl_err_number=PARSE_ERROR;
			$this->tpl_err_msg = "Alias '$alias' was not set, yet";
			$this->check_for_errors(); 
			old style, don't use it! */
		}
		/* check whether set_templates was made */
		if(!isset($this->tpl_filelist[$alias])) {
			$this->tpl_err_number=PARSE_ERROR;
			$this->tpl_err_msg = "Alias '$alias' is not known";
			$this->check_for_errors();						
		}
		/* check wheter file exists */
		$fn = $this->tpl_location . $this->tpl_filelist[$alias];
		if(!file_exists($fn)) {
			$this->tpl_err_number=FILE_NOT_FOUND;
			$this->tpl_err_msg = "Error while opening the file <b>$fn</b>";
			$this->check_for_errors();
		}
		$file = @fopen($fn, "r");
		if (!$file) {
			$this->tpl_err_number=FILE_NO_ACCESS;
			$this->tpl_err_msg = "File <b>$fn</b> cannot be opend";
			$this->check_for_errors();
		}
		$this->tpl_body="";
		while(!feof($file)) {
			$buffer = fgets($file);
			foreach($this->tpl_parse_vars[$alias] as $key => $value) {
				$buffer = ereg_replace("\{$key\}",  $value, $buffer);
			}
			$this->tpl_body.=$buffer;
		}
		@fclose($file);
		
		if ($clear) $this->tpl_parse_vars[$alias]=array();
	}

	/**
	*
	* This functions parses a tenplate by its alias and assigns the
	* parsed content into a new alias.
	*
	* $tpl->multiparse(array(array('alias_to_parse', 'alias_into_write', 'VARIABLE'),
	*			 ...,
	*			 array('alias_to_parse', 'alias_into_write', 'VARIABLE'))[,
	*			 array('last' => 1, 'alias_into_write')]));
	* @access	public
	* @see		parse
	*/

	function multiparse($array, $clear=1) {
		$this->clear_err_flags();
		if ($array == array())
		{
			$this->tpl_err_number=PARSE_ERROR;
			$this->tpl_err_msg = "This is an empty array";
			$this->check_for_errors();
		}

		/* Read the array */
		foreach($array as $parse => $value)
		{
			if(isset($value['last']) and $value['last']==1) {
				$this->parse($value[0], $clear);
			} else {

				$this->parse($value[0], $clear);
				$add = (isset($value[3]) and !$value[3]) ? 0 : 1;
				$this->assign($value[1], $value[2], $this->get_body(), $add);
			}
		}

		
		$this->check_for_errors();
	}
}

?>
